class Botomizer::Client
  def initialize(client, options = {})
    begin
      require "botomizer/#{client}".downcase

      @bot    = Object.const_get("Botomizer::#{client.capitalize}")
      @cache  = Botomizer::Cache.new(options[:cache] || nil, :botomizer_cmd)
      @name   = client
      @bot.options = options
      @bot.client = self
    rescue LoadError
      raise Botomizer::UnsupportedClient.new("Unsupported client #{client}")
    end
  end

  def name
    @name
  end

  def webhook
    @bot.webhook
  end

  def message
    bot.message
  end

  def chat
    bot.chat
  end

  def bot
    @bot
  end

  def cache
    @cache
  end

  def handle(request, handler)
    bot.parse_request(request)

    handler = handler.new(self)

    if message.cmd?
      @cache.write(chat.id, message.cmd)
    else
      message.cmd = @cache.fetch(chat.id)
    end

    if handler.respond_to? "cmd_#{message.cmd}"
      handler.public_send("cmd_#{message.cmd}")
    else
      handler.cmd_fallback
    end
  end

  def clean_history
    @cache.delete(chat.id)
  end

  def send(method, query = {})
    bot.send(method, query)
  end

  def send!(method, query = {})
    result = send(method, query)
    raise Botomizer::ResponseError.new(result[:message]) if result[:status] == :failed
    result
  end

  def send_message(text, options = {})
    result = bot.send_message(text, options)

    @cache.write("prev_#{chat.id}", result[:message_id]) if result.has_key?(:message_id)

    result
  end

  def send_message!(text, options = {})
    result = send_message(text, options)
    raise Botomizer::ResponseError.new(result[:message]) if result[:status] == :failed
    result
  end

  def delete_message(id = nil)
    bot.delete_message(id)
  end

  def delete_message!(id = nil)
    result = delete_message(id)
    raise Botomizer::ResponseError.new(result[:message]) if result[:status] == :failed
    result
  end

  def delete_previous_message
    if @cache.exists?("prev_#{chat.id}")
      delete_message(@cache.fetch("prev_#{chat.id}"))
    end
  end
end