module Botomizer
  # @abstact Exceptions raised by Botomizer inherit from Error
  class Error < StandardError; end

  # Exception raised when you attempt to set a non-existent client
  class UnsupportedClient < Error; end

  class ChatError < Error; end

  class MessageError < Error; end

  class BotError < Error; end

  # @abstract Exceptions which inherit from ResponseError contain the Net::HTTP
  # response object accessible via the {#response} method.
  class ResponseError < Error
    # Returns the response of the last request
    # @return [Net::HTTPResponse] A subclass of Net::HTTPResponse, e.g.
    # Net::HTTPOK
    attr_reader :response

    # Instantiate an instance of ResponseError with a Net::HTTPResponse object
    # @param [Net::HTTPResponse]
    def initialize(response)
      @response = response
      super(response)
    end
  end
end